var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { ConnectLogger } from "@amazon-connect/core";
import { AppContext } from "../app-context";
export class LifecycleManager {
    constructor(provider) {
        this.provider = provider;
        this.startHandlers = new Set();
        this.stopHandlers = new Set();
        this.state = { isRunning: false };
        this.isCreated = false;
        this.isDestroyed = false;
        this.logger = new ConnectLogger({
            source: "app.lifecycleManager",
            provider,
            mixin: () => ({
                state: this.state,
                isCreated: this.isCreated,
                isDestroyed: this.isDestroyed,
            }),
        });
    }
    handleLifecycleChangeMessage(msg) {
        const context = new AppContext(this.provider, msg.appInstanceId, msg.appConfig, msg.contactScope);
        this.state.appInstanceId = msg.appInstanceId;
        this.state.appConfig = msg.appConfig;
        this.state.contactScope = msg.contactScope;
        const params = { context };
        switch (msg.stage) {
            case "create":
                return this.handleCreate(params);
            case "start":
                return this.handleStart(params);
            case "stop":
                return this.handleStop(params);
            case "destroy":
                return this.handleDestroy(params);
        }
    }
    onStart(handler, options) {
        this.startHandlers.add(handler);
        // Allows subscribers to get start information even if they add their
        // subscription after the start event fires while app is running
        if (options === null || options === void 0 ? void 0 : options.invokeIfRunning) {
            if (this.state.isRunning) {
                // eslint-disable-next-line @typescript-eslint/no-floating-promises
                this.handleLifecycleChange(Object.assign(Object.assign({}, this.getLifecycleChangeParams()), { stage: "start" }), (e) => handler(e), false);
            }
        }
    }
    onStop(handler) {
        this.stopHandlers.add(handler);
    }
    offStart(handler) {
        this.startHandlers.delete(handler);
    }
    offStop(handler) {
        this.stopHandlers.delete(handler);
    }
    get appState() {
        return Object.assign({}, this.state);
    }
    handleCreate(params) {
        return __awaiter(this, void 0, void 0, function* () {
            var _a;
            if (this.isDestroyed) {
                this.logger.error("An attempt was Create after a Destroy. No Action", {
                    appInstanceId: params.context.appInstanceId,
                });
                return;
            }
            if (this.isCreated) {
                this.logger.error("An attempt was invoke Create after it was already invoked. No Action", { appInstanceId: params.context.appInstanceId });
                return;
            }
            this.logger.debug("Begin Lifecycle Create", {
                appInstanceId: params.context.appInstanceId,
            });
            if (!((_a = this.provider.config) === null || _a === void 0 ? void 0 : _a.onCreate)) {
                const msg = "App did not specify an onCreated handler. This is required. Closing app";
                const data = { appInstanceId: params.context.appInstanceId };
                this.logger.error(msg, { appInstanceId: params.context.appInstanceId });
                this.provider.sendFatalError(msg, data);
                return;
            }
            const { success } = yield this.handleLifecycleChange(Object.assign(Object.assign({}, params), { stage: "create" }), (e) => this.provider.config.onCreate(e), true);
            if (success) {
                this.isCreated = true;
                this.sendLifecycleHandlerCompletedMessage(params.context.appInstanceId, "create");
            }
        });
    }
    handleStart(params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDestroyed) {
                this.logger.error("An attempt was Start after a Destroy. No Action", {
                    appInstanceId: params.context.appInstanceId,
                });
                return;
            }
            if (!this.isCreated) {
                this.logger.error("An attempt was invoke Start before Create. No Action", { appInstanceId: params.context.appInstanceId });
                return;
            }
            this.state.isRunning = true;
            this.logger.info("Begin Lifecycle Start");
            const handlerRunResult = yield Promise.all([...this.startHandlers].map((h) => this.handleLifecycleChange(Object.assign(Object.assign({}, params), { stage: "start" }), (e) => h(e), false)));
            this.logger.debug("Completed all start handlers", {
                count: this.startHandlers.size,
                errorCount: handlerRunResult.filter(({ success }) => !success).length,
            });
        });
    }
    handleStop(params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDestroyed) {
                this.logger.error("An attempt was Stop after a Destroy. No Action", {
                    appInstanceId: params.context.appInstanceId,
                });
                return;
            }
            if (!this.isCreated) {
                this.logger.error("An attempt was invoke Stop before Create. No Action", {
                    appInstanceId: params.context.appInstanceId,
                });
                return;
            }
            this.state.isRunning = false;
            this.logger.info("Begin Lifecycle Stop");
            const handlerRunResult = yield Promise.all([...this.stopHandlers].map((h) => this.handleLifecycleChange(Object.assign(Object.assign({}, params), { stage: "stop" }), (e) => h(e), false)));
            this.logger.debug("Completed all stop handlers", {
                count: this.stopHandlers.size,
                errorCount: handlerRunResult.filter(({ success }) => !success).length,
            });
        });
    }
    handleDestroy(params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDestroyed) {
                this.logger.error("An attempt was invoke Destroy multiple times. No Action", {
                    appInstanceId: params.context.appInstanceId,
                });
                return;
            }
            if (!this.isCreated) {
                this.logger.error("An attempt was invoke Destroy before Create. No Action", {
                    appInstanceId: params.context.appInstanceId,
                });
                return;
            }
            this.isDestroyed = true;
            this.state.isRunning = false;
            this.logger.info("Begin Lifecycle Destroy");
            const { config } = this.provider;
            const { success } = yield this.handleLifecycleChange(Object.assign(Object.assign({}, params), { stage: "destroy" }), (e) => (config.onDestroy ? config.onDestroy(e) : Promise.resolve()), true);
            if (success) {
                this.sendLifecycleHandlerCompletedMessage(params.context.appInstanceId, "destroy");
            }
        });
    }
    handleLifecycleChange(evt, action, isFatal) {
        return __awaiter(this, void 0, void 0, function* () {
            let success = false;
            try {
                yield action(evt);
                success = true;
            }
            catch (error) {
                const { appInstanceId } = evt.context;
                if (isFatal) {
                    const msg = `An fatal error occurred when handling a ${evt.stage} lifecycle action. Closing app`;
                    this.logger.error(msg, { appInstanceId, error });
                    this.provider.sendFatalError(msg, error);
                }
                else {
                    this.logger.error(`An error occurred when handling a ${evt.stage} lifecycle action.`, {
                        appInstanceId,
                        error,
                    });
                }
            }
            return { success };
        });
    }
    getLifecycleChangeParams() {
        return {
            context: new AppContext(this.provider, this.state.appInstanceId, this.state.appConfig, this.state.contactScope),
        };
    }
    sendLifecycleHandlerCompletedMessage(appInstanceId, stage) {
        this.logger.debug(`Sending lifecycle ${stage} completed signal`);
        const proxy = this.provider.getProxy();
        proxy.sendLifecycleHandlerCompleted(appInstanceId, stage);
    }
}
//# sourceMappingURL=lifecycle-manager.js.map