import { ConnectLogger, Proxy, sdkVersion, TimeoutTracker, } from "@amazon-connect/core";
import { getConnectionTimeout } from "./connection-timeout";
export class AppProxy extends Proxy {
    constructor(provider, lifecycleManager) {
        super(provider);
        this.channel = new MessageChannel();
        this.lifecycleManager = lifecycleManager;
        this.appLogger = new ConnectLogger({
            source: "app.appProxy",
            provider: provider,
        });
    }
    get proxyType() {
        return "AppProxy";
    }
    sendLifecycleHandlerCompleted(appInstanceId, stage) {
        const msg = {
            type: "appLifecycleHandlerCompleted",
            stage,
            appInstanceId,
        };
        this.sendOrQueueMessageToSubject(msg);
    }
    tryCloseApp(message, isFatalError, data) {
        const msg = {
            type: "closeApp",
            isFatalError: isFatalError !== null && isFatalError !== void 0 ? isFatalError : false,
            message,
            data,
        };
        this.sendOrQueueMessageToSubject(msg);
    }
    publish(topic, data) {
        const msg = {
            type: "appPublish",
            topic,
            data,
        };
        this.sendOrQueueMessageToSubject(msg);
    }
    initProxy() {
        const hostInitMessage = {
            type: "connect-app-host-init",
            sdkVersion,
            providerId: this.provider.id,
        };
        this.status.update({ status: "initializing" });
        this.channel.port1.onmessage = (evt) => this.consumerMessageHandler(evt);
        this.connectionTimer = TimeoutTracker.start(this.connectionTimeout.bind(this), getConnectionTimeout(this.provider.config));
        window.parent.postMessage(hostInitMessage, "*", [this.channel.port2]);
        this.appLogger.debug("Send connect message to configure proxy");
    }
    sendMessageToSubject(message) {
        this.channel.port1.postMessage(message);
    }
    getUpstreamMessageOrigin() {
        if (document === null || document === void 0 ? void 0 : document.location) {
            const { origin, pathname: path } = document.location;
            return {
                _type: "app",
                providerId: this.provider.id,
                origin,
                path,
            };
        }
        else {
            return {
                _type: "app",
                providerId: this.provider.id,
                origin: "unknown",
                path: "unknown",
            };
        }
    }
    handleConnectionAcknowledge(msg) {
        // ConnectionTimer will always be defined here
        if (!this.connectionTimer.complete()) {
            this.appLogger.error("Workspace connection acknowledge received after timeout. App is not connected to workspace.", {
                timeout: this.connectionTimer.timeoutMs,
            });
            return;
        }
        super.handleConnectionAcknowledge(msg);
    }
    handleMessageFromSubject(msg) {
        switch (msg.type) {
            case "appLifecycle":
                this.lifecycleManager
                    .handleLifecycleChangeMessage(msg)
                    .catch((error) => {
                    this.appLogger.error("An error occurred when invoking handleLifecycleChangeMessage", { error });
                });
                break;
            default:
                super.handleMessageFromSubject(msg);
        }
    }
    connectionTimeout(evt) {
        this.status.update({
            status: "error",
            reason: "Workspace connection timeout",
            details: Object.assign({}, evt),
        });
        this.publishError({
            message: "App failed to connect to workspace in the allotted time",
            key: "workspaceConnectTimeout",
            details: Object.assign({}, evt),
            isFatal: true,
            proxyStatus: { initialized: false },
        });
    }
    addContextToLogger() {
        const { isRunning: appIsRunning } = this.lifecycleManager.appState;
        if (document === null || document === void 0 ? void 0 : document.location) {
            const { origin, pathname: path } = document.location;
            return { appIsRunning, app: { origin, path } };
        }
        else {
            return {
                appIsRunning,
                app: { origin: "unknown", path: "unknown" },
            };
        }
    }
}
//# sourceMappingURL=app-proxy.js.map